<?php
/**
 * @package    RO Payments
 *
 * @author     Roland Dalmulder <contact@rolandd.com>
 * @copyright  Copyright (C) 2009 - 2022 RolandD Cyber Produksi. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link       https://rolandd.com
 */

namespace Jdideal\Addons;

defined('_JEXEC') or die;

use JDatabaseDriver;
use Joomla\CMS\Factory;
use RuntimeException;
use stdClass;

/**
 * Event Booking addon.
 *
 * @package     JDIdeal
 * @subpackage  Addon
 * @since       2.13.0
 */
class AddonEventbooking implements AddonInterface
{
	/**
	 * Database interface
	 *
	 * @var    JDatabaseDriver
	 * @since  5.3.0
	 */
	private $db;

	/**
	 * Constructor.
	 *
	 * @since  5.3.0
	 */
	public function __construct()
	{
		$this->db = Factory::getDbo();
	}

	/**
	 * Get the name of the addon.
	 *
	 * @return  string  The name of the component.
	 *
	 * @since   5.3.0
	 */
	public function getName(): string
	{
		return 'Event Booking';
	}

	/**
	 * Collect the order information.
	 *
	 * @param   string  $orderId  The ID to get the order data for
	 * @param   array   $data     Array with payment info
	 *
	 * @return  array  Array with order information.
	 *
	 * @since   2.13
	 *
	 * @throws  RuntimeException
	 */
	public function getOrderInformation(string $orderId, array $data): array
	{
		$query = $this->db->getQuery(true)
			->select(
				$this->db->quoteName(
					[
						'email',
						'published',
						'total_amount',
					]
				)
			)
			->from($this->db->quoteName('#__eb_registrants'))
			->where($this->db->quoteName('id') . ' = ' . $this->db->q($orderId));
		$this->db->setQuery($query);
		$order = $this->db->loadObject();

		if (!$order)
		{
			$data['order_status'] = false;

			return $this->callBack($data);
		}

		$data['order_total']  = $order->total_amount;
		$data['order_status'] = $this->translateOrderStatus($order->published);
		$data['user_email']   = $order->email;

		// Return the data
		return $data;
	}

	/**
	 * Set the callback to go back to the component.
	 *
	 * @param   array  $data  The data from the processor
	 *
	 * @return  array  The order data.
	 *
	 * @since   2.13.0
	 */
	public function callBack(array $data): array
	{
		return $data;
	}

	/**
	 * Translate the order status from the component status to an RO Payments Gateway status.
	 *
	 * 1. Confirmed
	 * 2. Cancelled
	 * 3. Waiting list
	 *
	 * @param   string  $orderStatus  The code of the order status
	 *
	 * @return  string  the RO Payments order status code.
	 *
	 * @since   2.13.0
	 */
	public function translateOrderStatus(string $orderStatus): string
	{
		switch ($orderStatus)
		{
			case '2':
				return 'X';
			case '1':
				return 'C';
			case '3':
			default:
				return 'P';
		}
	}

	/**
	 * Get the customer information
	 *
	 * @param   string  $orderId  The order ID the request is for
	 *
	 * @return  array  Customer details.
	 *
	 * @since   2.13.0
	 *
	 * @throws  RuntimeException
	 */
	public function getCustomerInformation(string $orderId): array
	{
		// Collect the data
		$data = array();

		// Get the billing information
		$query = $this->db->getQuery(true)
			->select(
				$this->db->quoteName('first_name', 'firstname') . ','
				. $this->db->quoteName('last_name', 'lastname') . ','
				. $this->db->quoteName('organization', 'company') . ','
				. $this->db->quoteName('address', 'address1') . ','
				. $this->db->quoteName('address2', 'address2') . ','
				. $this->db->quoteName('city') . ','
				. $this->db->quoteName('zip') . ','
				. $this->db->quoteName('country') . ','
				. $this->db->quoteName('phone') . ','
				. $this->db->quoteName('email') . ','
				. $this->db->quoteName('amount') . ','
				. $this->db->quoteName('tax_amount', 'tax')
			)
			->from($this->db->quoteName('#__eb_registrants'))
			->where($this->db->quoteName('id') . ' = ' . $orderId);
		$this->db->setQuery($query);
		$data['billing'] = $this->db->loadObject();

		// Get the country information
		if (isset($data['billing']->country))
		{
			$query->clear()
				->select($this->db->quoteName('country_2_code'))
				->from($this->db->quoteName('#__eb_countries'))
				->where($this->db->quoteName('name') . ' = ' . $this->db->quote($data['billing']->country));
			$this->db->setQuery($query);
			$data['billing']->countrycode = $this->db->loadResult();
		}

		// Add the currency
		$data['currency'] = 'EUR';

		// Make shipping the same as billing
		$data['shipping'] = $data['billing'];

		return $data;
	}

	/**
	 * Get the order status name.
	 *
	 * @param   array  $data  Array with order information
	 *
	 * @return  string  The name of the new order status.
	 *
	 * @since   2.13.0
	 */
	public function getOrderStatusName(array $data): string
	{
		$result = '';

		switch ($data['order_status'])
		{
			case 'C':
				$result = '1';
				break;
			case 'P':
				$result = '0';
				break;
			case 'X':
				$result = '2';
				break;
		}

		return $result;
	}

	/**
	 * Get the component link.
	 *
	 * @return  string  The URL to the component.
	 *
	 * @since   2.13.0
	 */
	public function getComponentLink(): string
	{
		return 'index.php?option=com_eventbooking';
	}

	/**
	 * Get the administrator order link.
	 *
	 * @param   string  $orderId  The order ID for the link.
	 *
	 * @return  string  The URL to the order details.
	 *
	 * @since   2.13.0
	 */
	public function getAdminOrderLink(string $orderId): string
	{
		return 'index.php?option=com_eventbooking&view=registrant&cid[]=' . $orderId;
	}

	/**
	 * Get the order link.
	 *
	 * @param   string  $orderId      The order ID for the link
	 * @param   string  $orderNumber  The order number for the link
	 *
	 * @return  string  The URL to the order details.
	 *
	 * @since   2.13.0
	 */
	public function getOrderLink(string $orderId, string $orderNumber): string
	{
		return 'index.php?option=com_eventbooking&task=edit_registrant&from=histor&cid[]=' . $orderId;
	}

	/**
	 * Replace addon specific placeholders.
	 *
	 * @param   stdClass  $details  The transaction details
	 * @param   string    $message  The message to replace the placeholders for
	 *
	 * @return  string  The message with replaced placeholders.
	 *
	 * @since   5.3.0
	 */
	public function replacePlaceholders(stdClass $details, string $message): string
	{
		return $message;
	}
}
