<?php
/**
 * @package    JDiDEAL
 *
 * @author     Roland Dalmulder <contact@rolandd.com>
 * @copyright  Copyright (C) 2009 - 2022 RolandD Cyber Produksi. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link       https://rolandd.com
 */

defined('_JEXEC') or die;

use Jdideal\Gateway;
use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Table\Table;

/**
 * Event Booking payment plugin.
 *
 * @package  JDiDEAL
 * @since    3.0
 */
class os_jdideal extends os_payment
{
	/**
	 * Get the profile alias.
	 *
	 * @var    string
	 * @since  4.0
	 */
	private $profileAlias = '';

	/**
	 * Constructor functions, init some parameter
	 *
	 * @param   array  $params  The parameters for the payment plugin.
	 *
	 * @since   4.0
	 */
	public function __construct($params)
	{
		parent::setName('os_jdideal');
		parent::os_payment();
		parent::setCreditCard(false);
		parent::setCardType(false);
		parent::setCardCvv(false);
		parent::setCardHolderName(false);

		$this->profileAlias = $params->get('profileAlias');
	}

	/**
	 * Process the payment request.
	 *
	 * @param   object  $row   The payment details row.
	 * @param   array   $data  An array with payment information.
	 *
	 * @return string The HTML payment form.
	 *
	 * @since   3.0
	 *
	 * @throws  Exception
	 */
	public function processPayment($row, $data)
	{
		$jinput = Factory::getApplication()->input;
		$Itemid = $jinput->get('Itemid', 1, 'int');

		// Set some needed data
		$data = array(
			'amount'         => number_format($data['amount'], 2),
			'order_number'   => $row->transaction_id,
			'order_id'       => $row->id,
			'origin'         => 'eventbooking',
			'return_url'     => JUri::base()
				. 'index.php?option=com_eventbooking&view=complete&registration_code=' . $row->registration_code
				. '&Itemid=' . $Itemid,
			'notify_url'     => JUri::base()
				. 'index.php?option=com_eventbooking&task=payment_confirm&payment_method=os_jdideal',
			'cancel_url'     => JUri::base()
				. 'index.php?option=com_eventbooking&task=cancel&id=' . $row->id . '&Itemid=' . $Itemid,
			'email'          => '',
			'payment_method' => '',
			'profileAlias'   => $this->profileAlias,
			'custom_html'    => '',
			'silent'         => false,
		);

		// Build the form
		ob_start();

		// Include the payment form
		$layout = new JLayoutFile('forms.form', null, array('component' => 'com_jdidealgateway'));
		echo $layout->render(array('data' => $data));
		$html = ob_get_contents();

		ob_end_clean();

		echo $html;
	}

	/**
	 * Verify the payment.
	 *
	 * @return  void.
	 *
	 * @since   2.14
	 *
	 * @throws  Exception
	 * @throws  RuntimeException
	 * @throws  InvalidArgumentException
	 * @throws  UnexpectedValueException
	 */
	public function verifyPayment()
	{
		JLoader::registerNamespace('Jdideal', JPATH_LIBRARIES);

		if (class_exists(Gateway::class) === false)
		{
			JLoader::registerNamespace('Jdideal', JPATH_LIBRARIES . '/Jdideal');
		}

		$jinput  = Factory::getApplication()->input;
		$jdideal = new Gateway;

		$trans  = $jinput->get('transactionId');
		$column = 'trans';

		if (empty($trans))
		{
			$trans  = $jinput->get('pid');
			$column = 'pid';
		}

		$details = $jdideal->getDetails($trans, $column, false, 'eventbooking');

		$jdideal->log('Verify payment', $details->id);

		// Set the status
		switch ($jdideal->getStatusCode($details->result))
		{
			case 'C':
				$published = 1;
				break;
			case 'X':
				$published = 2;
				break;
			default:
				$published = 0;
				break;
		}

		// Update the registration
		$app    = Factory::getApplication('site');
		$config = EventbookingHelper::getConfig();
		$id     = $details->order_id;
		$amount = $details->amount;

		if ($amount < 0)
		{
			$jdideal->log('No amount. Stop processing.', $details->id);

			return false;
		}

		$jdideal->log('Load order ID: ' . $id, $details->id);

		$row = Table::getInstance('EventBooking', 'Registrant');
		$row->load($id);

		if (!$row->id)
		{
			return false;
		}

		$jdideal->log('Published status: ' . $row->published, $details->id);

		if ((int) $row->published === 1)
		{
			$jdideal->log('Order already published. Redirect to notify URL.', $details->id);

			$app->redirect($details->return_url);
		}
		else
		{
			$paymentPlugin = PluginHelper::getPlugin('system', 'jdidealeventbooking');
			$paymentParams = isset($paymentPlugin->params) ? json_decode($paymentPlugin->params) : new stdClass;

			if (isset($paymentParams->transactionId) && $paymentParams->transactionId === 'ropayments')
			{
				$row->transaction_id = $details->paymentId;
			}

			$row->payment_date = gmdate('Y-m-d H:i:s');
			$row->published    = $published;
			$jdideal->log('Store the order', $details->id);
			$row->store();

			if ($row->is_group_billing)
			{
				EventbookingHelper::updateGroupRegistrationRecord($row->id);
			}

			// All is good, return
			if ($published === 1)
			{
				PluginHelper::importPlugin('eventbooking');
				$dispatcher = JEventDispatcher::getInstance();
				$jdideal->log('Trigger plugins onAfterPaymentSuccess', $details->id);
				$dispatcher->trigger('onAfterPaymentSuccess', array($row));
				$jdideal->log('Payment is successful. Send emails.', $details->id);
				EventbookingHelper::sendEmails($row, $config);
				$jdideal->log('Redirect to return URL', $details->id);
				$app->redirect($details->return_url);
			}
			else
			{
				$jdideal->log('Payment not successful. Redirect to cancel URL.', $details->id);
				$app->redirect($details->cancel_url);
			}
		}
	}
}
