<?php
/**
 * @package     JDiDEAL
 * @subpackage  Eventbooking
 *
 * @author      Roland Dalmulder <contact@rolandd.com>
 * @copyright   Copyright (C) 2009 - 2022 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Language\Text;

/**
 * Load the RO Payments installer.
 *
 * @return  void.
 *
 * @since   3.3.0
 */
class PlgsystemjdidealeventbookingInstallerScript
{
	/**
	 * Run the preflight operations.
	 *
	 * @param   object  $parent  The parent class.
	 *
	 * @return  boolean  True on success | False on failure.
	 *
	 * @since   4.7.0
	 *
	 * @throws  Exception
	 */
	public function preflight($parent)
	{
		// Check if RO Payments is installed
		if ($parent === 'install' && ComponentHelper::isInstalled('com_jdidealgateway') === 0)
		{
			Factory::getApplication()->enqueueMessage(
				Text::_('PLG_EVENTBOOKING_ROPAYMENTS_ROPAYMENTS_NOT_INSTALLED'),
				'error'
			);

			return false;
		}

		// Check if extension is installed
		if (ComponentHelper::isInstalled('com_eventbooking') === 0)
		{
			Factory::getApplication()->enqueueMessage(
				Text::_('PLG_EVENTBOOKING_ROPPAYMENTS_EVENTBOOKING_NOT_INSTALLED'),
				'error'
			);

			return false;
		}

		return true;
	}

	/**
	 * Run the postflight operations.
	 *
	 * @param   object  $parent  The parent class.
	 *
	 * @return  boolean True on success | False on failure.
	 *
	 * @since   3.3.0
	 *
	 * @throws  Exception
	 */
	public function postflight($parent): bool
	{
		// Copy the files to the Eventbooking folder
		$src  = JPATH_SITE . '/plugins/system/jdidealeventbooking/files/';
		$dest = JPATH_SITE . '/components/com_eventbooking/payments';

		if (Folder::copy($src, $dest, '', true))
		{
			// Add the database entry
			$db       = Factory::getDbo();
			$row      = new stdClass;
			$pluginId = $this->getPluginId();

			// Get the largest ordering value for a given where clause.
			$query = $db->getQuery(true)
				->select('MAX(' . $db->quoteName('ordering') . ')')
				->from($db->quoteName('#__eb_payment_plugins'))
				->where($db->quoteName('published') . ' = 1');
			$db->setQuery($query);
			$row->ordering = (int) $db->loadResult() + 1;

			$row->name          = 'os_jdideal';
			$row->author        = 'RolandD Cyber Produksi';
			$row->creation_date = '2 November 2022';
			$row->copyright     = 'Copyright (C) 2009 - 2022 RolandD Cyber Produksi. All rights reserved.';
			$row->license       = 'GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html';
			$row->author_email  = 'contact@rolandd.com';
			$row->author_url    = 'https://rolandd.com';
			$row->version       = '8.0.3';
			$row->description   = 'RO Payments plugin for Event Booking';

			if ($pluginId)
			{
				$row->id = $pluginId;
				$db->updateObject('#__eb_payment_plugins', $row, 'id');
			}
			else
			{
				$row->title     = 'RO Payments';
				$row->published = 0;
				$db->insertObject('#__eb_payment_plugins', $row);
			}
		}

		return true;
	}

	/**
	 * Get the plugin ID.
	 *
	 * @return  integer  The ID of the plugin.
	 *
	 * @since   3.3.0
	 */
	private function getPluginId(): int
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
			->select($db->quoteName('id'))
			->from($db->quoteName('#__eb_payment_plugins'))
			->where($db->quoteName('name') . ' = ' . $db->quote('os_jdideal'));
		$db->setQuery($query);

		return (int) $db->loadResult();
	}

	/**
	 * Uninstall the plugin.
	 *
	 * @param   object  $parent  The parent class calling.
	 *
	 * @return  void
	 *
	 * @since   3.3
	 */
	public function uninstall($parent): void
	{
		// Remove the files from the Membership Pro folder
		$paths = [
			'/components/com_eventbooking/payments/os_jdideal.php',
			'/components/com_eventbooking/payments/os_jdideal.xml',
		];

		foreach ($paths as $path)
		{
			File::delete(JPATH_SITE . $path);
		}

		// Remove the database entry
		$pluginId = $this->getPluginId();

		if ($pluginId)
		{
			$db    = Factory::getDbo();
			$query = $db->getQuery(true)
				->delete($db->quoteName('#__eb_payment_plugins'))
				->where($db->quoteName('id') . ' = ' . (int) $pluginId);
			$db->setQuery($query)->execute();
		}

		// Remove the Eventbooking file
		File::delete(JPATH_ADMINISTRATOR . '/components/com_jdidealgateway/models/addons/eventbooking.php');
	}
}
